#include "PathfindingNode.h"
#include <algorithm>
PathfindingNode::PathfindingNode(const glm::vec3& initpos) : 
	pos(initpos), parent(nullptr), dist(0)
{}

void PathfindingNode::AddConnection(PathfindingNode* con) {
	if (con) {
		connections.push_back(con);
	}
}

void PathfindingNode::DoPath(PathfindingNode* dest) {
	// update distance and parent of connected nodes if applicable
	for (PathfindingNode* con : connections) {
		if (dist + 1 < con->dist) {
			con->dist = dist + 1;
			con->parent = this;
		}
	}
	if (!visited) {
		// we have been visited now
		visited = true;
		// visit each node in order of distance
		std::sort(connections.begin(), connections.end(), [](PathfindingNode * a, PathfindingNode * b) {
			return a->dist < b->dist;
		});
		// make them do the same (recurse)
		for (PathfindingNode* con : connections) {
			con->DoPath(dest);
			// we need to crash ASAP if there's a parent loop
			// obviously this shouldn't happen, but if it does, player's nodelist
			// will populate infinitely...
			assert(!(con == parent && con->parent == this));
		}
	}
}